﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;          // added for Serial port functions

namespace RadioShack_2200039
{
    public partial class RadioShack2200039 : Form
    {
        // ---- INITIALIZE GLOBAL VARIABLES ----
        string output;
        int packetOne, packetTwo, packetThree, packetFour, packetFive, packetSix, packetSeven, packetEight, packetNine;
        string packet4 = "", packet5 = "", packet6 = "", packet7 = "";
        int value;
        string fourDigitLCD;

        public RadioShack2200039()
        {
            InitializeComponent();
        }

        //  ============ function for serial port enumeration ================
        private void RadioShack2200039_Load(object sender, EventArgs e)
        {
            string[] ports = SerialPort.GetPortNames();
            multimeter_cmb.DataSource = ports;
        } // end of decimalToLCDValueFunc()

        
        //  =================== CONNECT BUTTON1, MULTIMETER 2200039 ===============
        private void connect_btn_Click(object sender, EventArgs e)
        {
            if (!myPort.IsOpen)
            {
                myPort = new SerialPort();
                myPort.BaudRate = 4800;
                myPort.PortName = multimeter_cmb.Text;
                myPort.Parity = Parity.None;
                myPort.DataBits = 8;
                myPort.StopBits = StopBits.One;
                myPort.Encoding = System.Text.Encoding.Default;

                myPort.DataReceived += myPort_DataReceived;     // calls function

                myPort.Open();

                connect_btn.Text = "Disconnect";
                connect_btn.BackColor = Color.IndianRed;
            }
            else
            {
                myPort.Close();
                connect_btn.Text = "Connect";
                connect_btn.BackColor = Color.White;
            }

        }                                          // end of CONNECT_BTN_CLICK

        //======== connect_btn DATA RECEIVED Function ========
        private void myPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            if (myPort.BytesToRead >= 9)    // do only once bytes are 9 or more
            {
                byte[] buf = new byte[9];                // create buf array of 9 entries
                myPort.Read(buf, 0, buf.Length);        // writes 9 bytes to buffer

                //grab packets as integers
                packetOne = buf[0];
                packetTwo = buf[1];
                packetThree = buf[2];
                packetFour = buf[3];        // beginig numbers on LCD
                packetFive = buf[4];
                packetSix = buf[5];
                packetSeven = buf[6];       // end of numbers on LCD
                packetEight = buf[7];
                packetNine = buf[8];

                foreach (int packet in buf)         // each packet is 8-bits, One Decimal number
                {
                    value = Convert.ToInt32(packet);                // converts packet to global 'value' int. Decimal representation of byte
                    output += value.ToString() + " ";                // concatenates 'value' to 'output' String and adds space
                }

                this.Invoke(new EventHandler(displaydata_event));     // after 9 bytes are saved, go to displaydata_event

            }       // END OF 'IF'  

        }   // end of myPort_DataReceived                                    

        void displaydata_event(object s, EventArgs e)
        {
            rawData_txtBox.AppendText(output + "\n");                               // write all 'output' in txtbox1
            rawData_txtBox.ScrollToCaret();
            output = "";

            // ====== MULTIMETER MODE DISPLAY ==========
            // data sheet found on : forums.parallax.com/attachment.php?attachmentid=88160&d=1325568007
            switch (packetOne)                                   // write to textBox
            {
                case 0:
                    packetOne_txtBox.Text = "DC V"; break;

                case 1:
                    packetOne_txtBox.Text = "AC V"; break;

                case 2:
                    packetOne_txtBox.Text = "DC uA"; break;

                case 8:
                    packetOne_txtBox.Text = "Ohms"; break;

                default:
                    packetOne_txtBox.Text = "error"; break;
            }

            switch (packetTwo)
            {
                case 1:                      // 0000 0010
                    LCDicon_txtBox1.Text = "V"; break;

                case 3:                      // 0000 0011
                    LCDicon_txtBox1.Text = "mV"; break;

                case 4:                      // 0000 0100
                    LCDicon_txtBox1.Text = "A"; break;

                case 5:                      //  0000 0101
                    LCDicon_txtBox1.Text = "mV"; break;

                case 32:                     // 0001 0000
                    LCDicon_txtBox1.Text = "M"; break;

                case 96:                     // 0110 0000
                    LCDicon_txtBox1.Text = "KOhms"; break;

                case 80:                     // 0101 0000
                    LCDicon_txtBox1.Text = "MOhms"; break;

                default:
                    LCDicon_txtBox1.Text = "error"; break;

            }

            // use function to convert decimal to integer string
            packet4 = decimalToLCDValueFunc(packetFour);      // packet4 = xxxx1, LCD1
            packet5 = decimalToLCDValueFunc(packetFive);      // packet 5 = xx1x  LCD2
            packet6 = decimalToLCDValueFunc(packetSix);       // packet 6 = x1xx  LCD3
            packet7 = decimalToLCDValueFunc(packetSeven);     // packet 7= 1xxx   LCD4 

            fourDigitLCD = "";                 // string
            fourDigitLCD += packet7;
            fourDigitLCD += packet6;
            fourDigitLCD += packet5;
            fourDigitLCD += packet4;
            fourDigitLCD = fourDigitLCD.Trim();

            // send to 7-seg screen and convert to double so comparison can be made
            sevenSegmentArray1.Value = fourDigitLCD;
            double tempMultiMeter = 0.0;
            try          // sometimes not a number!
            {
                tempMultiMeter = Convert.ToDouble(fourDigitLCD);
            }
            catch (Exception){}

     

        }                                   // end of displayData event

        // ======== decimal to LCD value converter function ========
        string decimalToLCDValueFunc(int decimalVal)   // INPUT:byte;decimal value -> OUTPUT: string value 
        {
            switch (decimalVal)
            {
                case 215:                           //1101 0111
                    return "0";
                case 223:                           // 1101 1111
                    return ".0";

                case 80:                             // 0101 0000
                    return "1";
                case 88:                             // 0101 1000
                    return ".1";

                case 181:                                // 1011 0101 
                    return "2";
                case 189:                                // 1011 1101 
                    return ".2";

                case 241:                              // 1111 0001 
                    return "3";
                case 249:                               // 1111 0001 
                    return ".3";

                case 114:                               // 0111 0010  
                    return "4";
                case 122:                                // 0111 1010   
                    return ".4";

                case 227:                                // 1110 0011  
                    return "5";
                case 235:                                   // 1110 0011   
                    return ".5";

                case 231:                                    // 1110 0111  
                    return "6";
                case 239:                                   // 1110 1111  
                    return ".6";

                case 81:                                     // 0101 0001  
                    return "7";
                case 89:                                     // 0101 1001 
                    return ".7";

                case 247:                                     // 1111 0111 
                    return "8";
                case 255:                                     // 1111 111  
                    return ".8";

                case 243:                                      //  1111 0011
                    return "9";
                case 251:                                     // 1111 1011
                    return ".9";

                default:
                    return "error";

            }   // end of switch

        }


    }
}
